% =========================================================================
% Prefix Notation
%       "u":    infomation bits                      ||Three combinations:
%       "c":    coded bits                           ||         uBit
%       "Bit":  bits                                 ||         cBit
%       "Sym":  Symbol: one spcace-time input block  ||         cSym
%       "Lr":   log-ratio probability
%       "  ":   probability
% Appendix Notation
%       "Likelihoods":  P(Y|X)
%       "Prior":        P(X)
%       "APP":          P(X|Y)     (TurboLike, cBitLrExtrinsic, infoBits,
%       SimuPara)
% =========================================================================
% Verify: results=ldpcDecoding2(cBitLrLikelihoods,[infoBits,zeros(1,1498)],Q1,Q2,N,J,K,M,N*J/K,iterLDPC,logInfinity);

function  [ben wen ber wer uBitLrExtrinsic] = LdpcDecoding(LDPC, cBitLrLikelihoods, infoBits, SimuPara)
global logInfinity
iterLDPC = SimuPara.iterTurbo;

N = LDPC.N; J = LDPC.J; K = LDPC.K; M = LDPC.M;
Q1 = LDPC.Q1; Q2=LDPC.Q2;
bitInQ2 = LDPC.bitInQ2;
checkInQ1= LDPC.checkInQ1;

ben = zeros(1,iterLDPC+1);
ber = zeros(1,iterLDPC+1);
wen = zeros(1,iterLDPC+1);
wer = zeros(1,iterLDPC+1);

rxInfoBits = round( (sign(cBitLrLikelihoods)+1)/2 );
ben(1) = biterr(infoBits,rxInfoBits(M+1:end)); 
if ben(1)==0, uBitLrExtrinsic = cBitLrLikelihoods;return;end
if ben(1)>0, wen(1) = 1;wer(1) = 1; ber(1) = ben(1)/length(infoBits); end   




LRpt = zeros(1,N);
LRft = cBitLrLikelihoods;
LRc2b = zeros(N*J/K,K);  % its size is same with that of LDPC.Q2
LRb2c = zeros(J ,N);  % its size is same with that of LDPC.Q1
for iter=2:iterLDPC+1
    %bit-to-check messages:LRb2ctl
    LRb2c = repmat(LRft + sum(LRc2b(bitInQ2),1), J, 1)  - LRc2b(bitInQ2);
    LRb2c = dataBounded(LRb2c,logInfinity);
    
    %check-to-bit message:LRc2btl
    tempValue = f( abs(LRb2c(checkInQ1)) );
    tempSign  =   sign(LRb2c(checkInQ1));
    LRc2b = repmat(prod(tempSign,2),1,K)./tempSign .*  f(repmat(sum(tempValue,2),1,K) - tempValue);
    LRc2b = dataBounded(LRc2b,logInfinity);

    LRpt = LRft + sum(LRc2b(bitInQ2),1);
    LRpt = dataBounded(LRpt,logInfinity);

    rxBits  = zeros(1,length(LRpt));
    rxBits(LRpt>0)  = 1;
    ben(iter) = biterr(infoBits,rxBits(M+1:end)); 
    ber(iter) = ben(iter)/length(infoBits); 
    if ben(iter)==0,break;end
    if ben(iter) > 0, wen(iter) = 1;wer(iter) = 1;end
end
if nargout == 5
    uBitLrExtrinsic = LRpt - cBitLrLikelihoods;
end

function result=f(x)
result = log((exp(x)+1)./(exp(x)-1));
