% The function is to transfer the log-ratio soft information for bit to that for symbol

% KK:               how many bits are regarded as one symbol

% bitLrLikelihoods:   Input: The first row is log( P(bit = 1)/P(bit = 0)))
%                     Add one row with all "0": log( P(bit = 0)/P(bit = 0))

% symLrLikelihoods:   The first row is  log( P(symbol = 0)/P(symbol = 0))
%                   The second row is log( P(symbol = 1)/P(symbol = 0))
%                                 ...
%                   The last row is   log( P(symbol = 2^(KK-1)/P(symbol = 0))

function symLrLikelihoods = lrSoftBit2Symbol(KK,bitLrLikelihoods)
if KK == 1, symLrLikelihoods = bitLrLikelihoods(end,:); return; end;

[ModuSize bitLength] = size(bitLrLikelihoods);
if ModuSize ==1
    ModuSize = 2;
    bitLrLikelihoods = [zeros(1,size(bitLrLikelihoods,2));bitLrLikelihoods];
elseif ModuSize == 2
    if ~isempty( find(bitLrLikelihoods(1,:) ~= 0))
        error('The first row of input should be 0, i.e. log( P(bit = 0)/P(bit = 0))');
    end
else
    %This function can only deal with the binary inputs
    error('The input should have 1 or 2 rows');
    error('This function can only deal with the binary inputs');
end

symLength = bitLength/KK;
symLrLikelihoods = zeros(2^KK-1, symLength);
for symbolIndex = 1:2^KK-1
    % How to express the symbol in bits
    symInBits = de2bi(symbolIndex, KK, ModuSize, 'left-msb');
    
    % Find position where the required Pr(bit) appear in bitLrLikelihoods
    position = false(ModuSize, KK);
    rows = symInBits+1;cols=[1:KK];
    position( sub2ind(size(position), rows, cols ) ) = logical(1);
    position = repmat(position,1,symLength);
    
    symLrLikelihoods(symbolIndex, :) = sum(reshape(bitLrLikelihoods(position), KK,[]),1);        
end
