% ==========================================================================
% Onjective: 
% The function is to transfer the log-ratio soft information for bit to that for symbol
%       In general, the input "symLrLikelihoods" has 2^(KK-1) rows, we will 
%       append a all-zero row in front of symLrLikelihoods.
%       However, the input might has 2^KK rows. The 1st rwo should be all-0.
% ==========================================================================
% Input Parameters:
%       KK:               
%           how many bits are regarded as one symbol
%       symLrLikelihoods: 
%           The first row is  log( P(symbol = 0)/P(symbol = 0))
%           The second row is log( P(symbol = 1)/P(symbol = 0))
%                                 ...
%           The last row is   log( P(symbol = 2^(KK-1)/P(symbol =0))
% ==========================================================================
% Output Parameters:
%       bitLrLikelihoods:   One row: log( P(bit = 1)/P(bit = 0))
% ==========================================================================

function bitLrLikelihoods = lrSoftSymbol2Bit(KK,symLrLikelihoods,algLogSumExp)
if KK==1, bitLrLikelihoods = symLrLikelihoods(end,:); return; end;

if nargin == 2, algLogSumExp = 'EXACT';end
symLength = size(symLrLikelihoods,2);

ModuSize = 2;
if size(symLrLikelihoods,1) == ModuSize^KK - 1
    symLrLikelihoods = [zeros(1,size(symLrLikelihoods,2));symLrLikelihoods];
elseif size(symLrLikelihoods,1) == ModuSize^KK
    if ~isempty( find(symLrLikelihoods(1,:) ~= 0))
        error('Input has 2^kk rows. The first row should be 0, i.e. log( P(symbol = 0)/P(symbol = 0))');
    end
else
    error('The number of row should be 2^kk or 2&(kk-1)');
end

bitLength = symLength*KK;
bitLrLikelihoods = zeros(1, bitLength);

% How to express the symbol in bits
symInBits = de2bi([0:ModuSize^KK-1], KK, ModuSize, 'left-msb');
for bitIndex = 0:KK-1
    % Find position where the bitIndex-th bit appears in symLrLikelihoods
    posBit0 = find(symInBits(:,bitIndex+1)==0);
    posBit1 = find(symInBits(:,bitIndex+1)==1);
    bitLrLikelihoods(bitIndex+1+[0:symLength-1]*KK) = ...
        LogSumExp(symLrLikelihoods(posBit1,:),1,algLogSumExp) - ...
        LogSumExp(symLrLikelihoods(posBit0,:),1,algLogSumExp);
end

