%mex -g ldpcDecoding.c
%mex -g ldpcDecoding2.c
clear all;
%=====================================================================================
%============================ Global Simulation Parameters============================
%=====================================================================================
rand('state',5701);
randn('state',1);
global logInfinity
logInfinity  = 64;       % log(1/eps) = 36.044

%=====================================================================================
%================================= Modulation Parameters =============================
% 8PSK: 'Natural', 'Gray', 'd21', 'd23', 'AntiGray', 'Arbitary', 'QAM'
% 4PSK: 'Natural', 'Gray'
% 2PSK: 'Natural'
ModuScheme = struct('moduMethod','qask/arb',...
                    'moduMethod2','PSK',...
                    'moduSize',4,'mapRule','Gray');
ModuScheme = setModuParameter(ModuScheme); % 'PSK'  or 'QAM'

% type: % 'AWGN','FastFading', or 'SlowFading'
CHANNEL = struct('type','FastFading','Nt',2,'Nr',2);

SimuPara = struct('iterSuper',20,'iterTurbo',1,...
                  'simErrWordLimit',100,'simAllWordLimit',50000);
              
% needNewGs = YES (Always generate new Hs and Gs.) 
%           = NO  (Don't generate new Hs & Gs if they already exist)
LDPCC = struct('scheme','LDPC','NJK',[1024 3 4],'needNewGs','NO');
[LDPCC,SimuPara] = setLDPCCParameter(LDPCC,SimuPara);

    
R   =   ModuScheme.rate * LDPCC.rate * CHANNEL.Nt;
T   =   round(SimuPara.infoBitsLength  / R);  % the total number of Channel Use
EbNoR_dB=[-2:0.5:0.5]';
%=====================================================================================
%=======================================Main Routine==================================
%=====================================================================================
bitSent = zeros(1,length(EbNoR_dB));
wrdSent = zeros(1,length(EbNoR_dB));
berSum  = zeros(length(EbNoR_dB), SimuPara.iterTurbo+1, SimuPara.iterSuper);  %the error after i'th iteration
benSum  = zeros(length(EbNoR_dB), SimuPara.iterTurbo+1, SimuPara.iterSuper);  % From 0-th iterations  
wenSum  = zeros(length(EbNoR_dB), SimuPara.iterTurbo+1, SimuPara.iterSuper);
werSum  = zeros(length(EbNoR_dB), SimuPara.iterTurbo+1, SimuPara.iterSuper);
for EbNoR_n = 1:length(EbNoR_dB)
    EbNoR=10.^(EbNoR_dB(EbNoR_n)/10); EcuNoR=R*EbNoR; EsNoR=EcuNoR/CHANNEL.Nt;
    Es=1;Ecu=Es*CHANNEL.Nt; Eb=Ecu/R;  % Ecu: Enery/channel use
    No      = Es/EsNoR;
    No= No;
    
    while wenSum(EbNoR_n,end) < SimuPara.simErrWordLimit && wrdSent(EbNoR_n) < SimuPara.simAllWordLimit
        wrdSent(EbNoR_n) = wrdSent(EbNoR_n) + 1;   
        bitSent(EbNoR_n) = bitSent(EbNoR_n) + SimuPara.infoBitsLength;   
        
        infoBits    = gnInfoBits(SimuPara.infoBitsLength);

        [txBits LDPCC]= outCoding(LDPCC,infoBits);
        
        txSymbols   = gnSymbol(txBits, ModuScheme);

        txSignal    = gnTxSignal(txSymbols,ModuScheme,CHANNEL,Es); % [Nt x T]
        
        H           = gnFading(CHANNEL, T);

        rxSignal    = passChannel(txSignal, CHANNEL, H, No); % noise added.
        
        % Symbol (cSymLikelihoods): generate likelihood information for each Nt symbols
        cSymLrLikelihoods = gnLikelihood(rxSignal, LDPCC, CHANNEL, ModuScheme, Es, H, No);
        
        [ben wen ber wer] = outDecoding(LDPCC, ModuScheme.rate*CHANNEL.Nt, cSymLrLikelihoods, infoBits, SimuPara);

        benSum(EbNoR_n,:,:) = benSum(EbNoR_n,:,:) + reshape(ben,size(benSum(EbNoR_n,:,:))); % when the last dimention is 1, it will be ignored by Matlab. col vector=>row vector
        berSum(EbNoR_n,:,:) = benSum(EbNoR_n,:,:)/wrdSent(EbNoR_n)/SimuPara.infoBitsLength;
        wenSum(EbNoR_n,:,:) = wenSum(EbNoR_n,:,:) + reshape(wen,size(benSum(EbNoR_n,:,:)));
        werSum(EbNoR_n,:,:) = wenSum(EbNoR_n,:,:)/wrdSent(EbNoR_n);

        dispResults(EbNoR_n, EbNoR_dB, bitSent, benSum, berSum, wrdSent, wenSum, werSum, SimuPara);            
        save midResult.mat EbNoR_n EbNoR_dB bitSent benSum berSum wrdSent wenSum werSum SimuPara; 
    end
end



