% =========================================================================
% Prefix Notation
%       "u":    infomation bits                      ||Three combinations:
%       "c":    coded bits                           ||         uBit
%       "Bit":  bits                                 ||         cBit
%       "Sym":  Symbol: one spcace-time input block  ||         cSym
%       "Lr":   log-ratio probability
%       "  ":   probability
% Appendix Notation
%       "Likelihoods":  P(Y|X)
%       "Prior":        P(X)
%       "APP":          P(X|Y)
% =========================================================================
% Input:
%       numBitsInSymbol: the number of bits that compose one symbol
% =========================================================================
function [benSum wenSum berSum werSum] = outDecoding(LDPCC, numBitsInSymbol, cSymLrLikelihoods, infoBits, SimuPara)
global logInfinity

benSum = zeros(SimuPara.iterTurbo+1, SimuPara.iterSuper);
wenSum = zeros(SimuPara.iterTurbo+1, SimuPara.iterSuper);
berSum = zeros(SimuPara.iterTurbo+1, SimuPara.iterSuper);
werSum = zeros(SimuPara.iterTurbo+1, SimuPara.iterSuper);

cBitLrPriors = zeros(1,size(cSymLrLikelihoods,2)*numBitsInSymbol);
for iterSuper = 1: SimuPara.iterSuper
    % Soft-Demapper: spce-time input symbol-block ==>   input bit
    cSymLrPriors       = lrSoftBit2Symbol(numBitsInSymbol,cBitLrPriors);
    cSymLrAPP          = cSymLrPriors + cSymLrLikelihoods;
    cBitLrAPP          = lrSoftSymbol2Bit(numBitsInSymbol,cSymLrAPP);

    % "cBitLrExtrinsic" will be regarded as "cSymLikelihoods" in the inner interation
    cBitLrExtrinsic    = cBitLrAPP - cBitLrPriors;
    cBitLrExtrinsic    = dataBounded(cBitLrExtrinsic,logInfinity);
    
    [ben wen ber wer cBitLrExtrinsic] = LdpcDecoding(LDPCC, cBitLrExtrinsic, infoBits, SimuPara);

    cBitLrPriors = cBitLrExtrinsic;
    
    benSum(:,iterSuper)=ben;
    berSum(:,iterSuper)=ber;
    wenSum(:,iterSuper)=wen;
    werSum(:,iterSuper)=wer;
    if benSum(end,iterSuper) == 0,return;end
end

