function ModuScheme = setModuParameter(ModuScheme)

ModuScheme.rate = log2(ModuScheme.moduSize);

if strcmp(ModuScheme.moduMethod2, 'QAM')
    if ModuScheme.moduSize==8
        mapping = [4 6 7 5 3 1 0 2];
    elseif ModuScheme.moduSize==16
        mapping = [9 10 6 5 4 8 12 13 14 15 11 7 3 2 1 0];
    end
    points = qammod(mapping,ModuScheme.moduSize);
elseif strcmp(ModuScheme.moduMethod2, 'PSK')
    mapping = [];
    if ModuScheme.moduSize==2
        ;
    elseif ModuScheme.moduSize==4
        if strcmp(ModuScheme.mapRule, 'Gray')
            mapping = [0 1 3 2];
        end
    elseif ModuScheme.moduSize==8
        if strcmp(ModuScheme.mapRule, 'Gray')
            mapping = [0 1 3 2 7 6 4 5];
        elseif strcmp(ModuScheme.mapRule, 'd21')
            mapping = [0 5 6 1 7 2 3 4];
        elseif strcmp(ModuScheme.mapRule, 'd23')
            mapping = [0 4 5 1 6 2 3 7];
        elseif strcmp(ModuScheme.mapRule, 'AntiGray')
            mapping = [0 2 6 4 5 7 3 1];
        elseif strcmp(ModuScheme.mapRule, 'Arbitary')
            mapping = ranperm(ModuScheme.moduSize)-1;
            mapping = [ 0 7 2 6 4 5  3  1];
        end
    end

    if isempty(mapping) 
        if ~strcmp(ModuScheme.mapRule, 'Natural')
            ModuScheme.mapRule = 'Natural';
            disp('This kind of mapping rule is not supported. Natural Mapping will be adopted.');
        end
        mapping = [0:ModuScheme.moduSize-1];
    end
    points = pskmod(mapping,ModuScheme.moduSize); % Vector of all points in constellation
end

%Normalize the constellation points
points = points/sqrt(points*points'/length(points));

ModuScheme.inphase = real(points);
ModuScheme.quadr   = imag(points);

ModuScheme.inphase(abs(ModuScheme.inphase)<1e-10) = 0;
ModuScheme.quadr(abs(ModuScheme.quadr)<1e-10) = 0;


