% ==========================================================================
% Onjective: 
% The function is to transfer the log-ratio soft information for bit to that for symbol
%       In general, the input "bitLrLikelihoods" has 1 row, we will 
%           append a all-zero row in front of it.
%       If the input might has 2 rows, the 1st rwo should be all-0.
% ==========================================================================
% Input Parameters:
%       KK:   how many bits are regarded as one symbol
%       bitLrLikelihoods:   
%            1st row: log( P(bit = 0)/P(bit = 0)) = 0
%            2nd row: log( P(bit = 1)/P(bit = 0))
% ==========================================================================
% Output Parameters:
%       symLrLikelihoods has 2^(KK-1) rows
%           The first row is  log( P(symbol = 1)/P(symbol = 0))
%           The second row is log( P(symbol = 2)/P(symbol = 0))
%                                 ...
%           The last row is   log( P(symbol = 2^(KK-1)/P(symbol =0))
% ==========================================================================

function symLikelihoods = softBit2Symbol(KK,bitLikelihoods)
if KK==1, symLikelihoods = bitLikelihoods; return; end;

% Normailize it
bitLikelihoods = bitLikelihoods./ repmat(sum(bitLikelihoods,1),size(bitLikelihoods,1),1);

% how many symbols are invovled
[ModuSize bitLength] = size(bitLikelihoods);
%This function can only deal with the binary inputs
if ModuSize ~= 2, error('This function can only deal with the binary inputs');end;
if 2^KK == ModuSize, symLikelihoods = bitLikelihoods; return; end;

symLength = bitLength/KK;
symLikelihoods = zeros(2^KK, symLength);
for symbolIndex = 0:2^KK-1
    % How to express the symbol in bits
    symInBits = de2bi(symbolIndex, KK, ModuSize, 'left-msb');
    
    % Find position that the required Pr(bit) appear in bitLikelihoods
    position = false(ModuSize, KK);
    rows = symInBits+1;cols=[1:KK];
    position( sub2ind(size(position), rows, cols ) ) = logical(1);
    position = repmat(position,1,symLength);
    
    symLikelihoods(symbolIndex+1, :) = prod(reshape(bitLikelihoods(position), KK,[]),1);        
end



